# Copyright (C) 2013 Linaro
#
# Author: James Tunnicliffe <james.tunnicliffe@linaro.org>
#
# This file is part of Linaro CI CLI.
#
# Linaro CI CLI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linaro CI CLI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linaro Image Tools.  If not, see <http://www.gnu.org/licenses/>.

import os
import requests
import sys
from urlparse import urljoin


class BaseCommand(object):
    def __init__(self, settings, name):
        self.settings = settings
        self.name = name

    def _prep_request(self, url, in_params=None):
        params = {"username": self.settings["username"],
                  "api_key": self.settings["api_key"],
                  "format": "json"}

        url = urljoin(self.settings["ci_server"],
                      self.settings["api_prefix"] + url)

        # If we don't have a trailing slash, we will just be redirected. Avoid
        # the extra call.
        if url[-1] != "/":
            url += "/"

        if in_params:
            params.update(in_params)

        return url, params

    def get(self, url, in_params=None):
        url, params = self._prep_request(url, in_params)
        return requests.get(url, params=params)

    def post(self, url, data, in_params=None):
        url, params = self._prep_request(url, in_params)
        return requests.post(url, data=data, params=params,
                             headers={"content-type": "application/json"})

    @property
    def program_name(self):
        return '%s %s' % (os.path.basename(sys.argv[0]), self.name.lower())


class BaseCommandNeedsAuth(BaseCommand):
    def __init__(self, settings, name):
        import login

        super(BaseCommandNeedsAuth, self).__init__(settings, name)

        # Always run login. It won't ask for login details unless it needs them
        login_command = login.Command(settings, name)
        login_command.run(None)


class CommandFailed(Exception):
    def __init__(self, command, params, response):
        self.command = command
        self.params = params
        self.response = response

    def __str__(self):
        message = "An error occurred when executing the command %s" %\
                  self.command
        message += "\nURL %s\nReason %s\nHeaders %s\n\n%s" % (
            self.response.url,
            self.response.reason,
            self.response.headers,
            self.response.text,
        )

        return message
