# Copyright (C) 2013 Linaro
#
# Author: James Tunnicliffe <james.tunnicliffe@linaro.org>
#
# This file is part of Linaro CI CLI.
#
# Linaro CI CLI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linaro CI CLI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linaro Image Tools.  If not, see <http://www.gnu.org/licenses/>.

from copy import copy
import unittest
import urlparse
from commands.base_command import (
    BaseCommand,
    BaseCommandNeedsAuth,
)
from commands.defaults import (
    CI_SERVER,
    API_PREFIX,
)
import commands.login
from mock import patch
from test_utilities import FakeGot, expectedUrl

class TestBaseCommand(unittest.TestCase):
    def setUp(self):
        self.login_details = {"username": "me",
                              "api_key": "12qwaszx"}
        self.settings = {"ci_server": CI_SERVER,
                         "api_prefix": API_PREFIX}
        self.settings.update(self.login_details)

    @patch('requests.get')
    def test_get(self, mock_requests):
        cmd = BaseCommand(self.settings, "test_get")
        location = "fetch"

        cmd.get(location)

        self.assertEqual(expectedUrl(location),
                         mock_requests.call_args[0][0])
        self.assertDictContainsSubset(self.login_details,
                                      mock_requests.call_args[1]["params"])


class TestBaseCommandNeedsAuth(unittest.TestCase):
    def setUp(self):
        # Monkey patch the raw_input function imported by login
        self.fake_input = self.FakeRawInputLogin()
        commands.login.raw_input = self.fake_input.raw_input_response

        self.login_details = {"username": "me",
                              "api_key": "12qwaszx"}
        self.server_details = {"ci_server": CI_SERVER,
                               "api_prefix": API_PREFIX}
        self.settings = copy(self.server_details)
        self.settings.update(self.login_details)

    def tearDown(self):
        # Restore the original raw_input
        commands.login.raw_input = raw_input

    class FakeGot(object):
        def __init__(self, return_codes):
            self.return_codes_index = 0
            self.return_codes = return_codes

        @property
        def status_code(self):
            if isinstance(self.return_codes, list):
                return_code = self.return_codes[self.return_codes_index]
                self.return_codes_index += 1
                return return_code
            else:
                return self.return_codes

    @patch('requests.get')
    def test_logged_in(self, mock_requests):
        mock_requests.return_value = FakeGot(200)

        # By creating a BaseCommandNeedsAuth, login is triggered. We have
        # already got a username and api_key so BaseCommandNeedsAuth will test
        # the login. We fake the status code above so this call passes.
        BaseCommandNeedsAuth(copy(self.settings), "test_logged_in")

        self.assertEqual(expectedUrl("login_test"),
                         mock_requests.call_args[0][0])
        self.assertDictContainsSubset(self.login_details,
                                      mock_requests.call_args[1]["params"])

    class FakeRawInputLogin(object):
        def raw_input_response(self, prompt):
            if prompt == 'User ID: ':
                return "me"
            elif prompt == 'API Key: ':
                return "12qwaszx"
            else:
                assert False, "Unknown prompt"

    @patch('requests.get')
    def test_login(self, mock_requests):
        mock_requests.return_value = FakeGot(200)

        # By creating a BaseCommandNeedsAuth, login is triggered. We pass an
        # empty username and api_key, forcing a prompt for those values.
        BaseCommandNeedsAuth(self.server_details, "test_login")

        self.assertEqual(expectedUrl("login_test"),
            mock_requests.call_args[0][0])
        self.assertDictContainsSubset(self.login_details,
            mock_requests.call_args[1]["params"])

    @patch('requests.get')
    def test_login_update_details(self, mock_requests):
        mock_requests.return_value = FakeGot([401, 200])

        # Create some "old" login details with an invalid API key
        old_login_details = copy(self.login_details)
        old_login_details["api_key"] = "expired"

        old_settings = copy(self.server_details)
        old_settings.update(old_login_details)

        # By creating a BaseCommandNeedsAuth, login is triggered. We fake the
        # first login attempt so it fails, so we should have two prompts.
        BaseCommandNeedsAuth(copy(old_settings), "test_login_update_details")

        # First login attempt should be with the old settings
        self.assertEqual(expectedUrl("login_test"),
            mock_requests.call_args_list[0][0][0])
        self.assertDictContainsSubset(old_login_details,
            mock_requests.call_args_list[0][1]["params"])

        # Second login attempt should be with the new settings
        self.assertEqual(expectedUrl("login_test"),
            mock_requests.call_args_list[1][0][0])
        self.assertDictContainsSubset(self.login_details,
            mock_requests.call_args_list[1][1]["params"])

if __name__ == '__main__':
    unittest.main()
